#!/usr/bin/env python3
"""
Trojan Detector
Advanced malware and trojan detection tool.
Parameters:
    - path: Directory or file path to scan
    - rules: Path to YARA rules file
    - deep: Enable deep scan mode
Requires: yara-python, psutil packages and admin/root privileges
"""
import yara
import psutil
import os
import argparse
import logging
from datetime import datetime
import json
import threading
import queue
import hashlib
import magic
import re
from pathlib import Path

class TrojanDetector:
    def __init__(self, scan_path, rules_path=None, deep_scan=False):
        self.scan_path = Path(scan_path)
        self.rules_path = rules_path
        self.deep_scan = deep_scan
        self.results_queue = queue.Queue()
        self.suspicious_patterns = self.compile_patterns()
        self.setup_logging()
        self.load_yara_rules()

    def setup_logging(self):
        logging.basicConfig(
            level=logging.INFO,
            format='%(asctime)s - %(levelname)s - %(message)s'
        )
        self.logger = logging.getLogger(__name__)

    def compile_patterns(self):
        return {
            'network': re.compile(
                rb'(socket\.|connect\(|bind\(|listen\(|accept\(|recv\(|send\()'
            ),
            'process': re.compile(
                rb'(CreateProcess|ShellExecute|WinExec|system\(|popen\(|exec\()'
            ),
            'registry': re.compile(
                rb'(RegCreate|RegOpen|RegSet|RegGet|RegDelete)'
            ),
            'file_ops': re.compile(
                rb'(CreateFile|WriteFile|CopyFile|DeleteFile|MoveFile)'
            ),
            'encryption': re.compile(
                rb'(CryptoAPI|Crypt|AES|DES|RC4|MD5|SHA)'
            ),
            'persistence': re.compile(
                rb'(Run|RunOnce|Schedule|Task|Service|Driver)'
            ),
            'anti_debug': re.compile(
                rb'(IsDebugger|CheckRemoteDebugger|OutputDebug)'
            )
        }

    def load_yara_rules(self):
        try:
            if self.rules_path:
                self.rules = yara.compile(self.rules_path)
            else:
                # Default basic rules if no rules file provided
                rules_text = """
                rule Suspicious_Strings {
                    strings:
                        $s1 = "cmd.exe" nocase
                        $s2 = "powershell" nocase
                        $s3 = "wget" nocase
                        $s4 = "curl" nocase
                        $s5 = "nc " nocase
                        $s6 = "netcat" nocase
                        $s7 = "mimikatz" nocase
                        $s8 = "psexec" nocase
                    condition:
                        any of them
                }
                
                rule Suspicious_Behaviors {
                    strings:
                        $b1 = "CreateRemoteThread" nocase
                        $b2 = "VirtualAlloc" nocase
                        $b3 = "WriteProcessMemory" nocase
                        $b4 = "CreateProcess" nocase
                        $b5 = "WinExec" nocase
                    condition:
                        2 of them
                }
                
                rule Encoded_Content {
                    strings:
                        $enc1 = /base64_decode|base64_encode/ nocase
                        $enc2 = /eval\(|exec\(/ nocase
                        $enc3 = "FromBase64String" nocase
                    condition:
                        any of them
                }
                """
                self.rules = yara.compile(source=rules_text)
        except Exception as e:
            self.logger.error(f"Error loading YARA rules: {str(e)}")
            self.rules = None

    def calculate_file_hash(self, file_path):
        """Calculate MD5, SHA1, and SHA256 hashes of a file"""
        try:
            md5 = hashlib.md5()
            sha1 = hashlib.sha1()
            sha256 = hashlib.sha256()
            
            with open(file_path, 'rb') as f:
                while True:
                    data = f.read(65536)  # Read in 64kb chunks
                    if not data:
                        break
                    md5.update(data)
                    sha1.update(data)
                    sha256.update(data)
            
            return {
                'md5': md5.hexdigest(),
                'sha1': sha1.hexdigest(),
                'sha256': sha256.hexdigest()
            }
        except Exception as e:
            self.logger.error(f"Error calculating hash for {file_path}: {str(e)}")
            return None

    def check_file_type(self, file_path):
        """Determine file type using magic numbers"""
        try:
            return magic.from_file(str(file_path))
        except Exception as e:
            self.logger.error(f"Error determining file type for {file_path}: {str(e)}")
            return "Unknown"

    def scan_file_patterns(self, file_path):
        """Scan file for suspicious patterns"""
        suspicious_matches = []
        try:
            with open(file_path, 'rb') as f:
                content = f.read()
                for pattern_name, pattern in self.suspicious_patterns.items():
                    matches = pattern.findall(content)
                    if matches:
                        suspicious_matches.append({
                            'pattern': pattern_name,
                            'matches': [m.decode(errors='ignore') for m in matches]
                        })
        except Exception as e:
            self.logger.error(f"Error scanning patterns in {file_path}: {str(e)}")
        return suspicious_matches

    def scan_file(self, file_path):
        try:
            file_info = {
                'path': str(file_path),
                'size': os.path.getsize(file_path),
                'created': datetime.fromtimestamp(
                    os.path.getctime(file_path)
                ).isoformat(),
                'modified': datetime.fromtimestamp(
                    os.path.getmtime(file_path)
                ).isoformat(),
                'type': self.check_file_type(file_path),
                'hashes': self.calculate_file_hash(file_path),
                'suspicious_patterns': [],
                'yara_matches': [],
                'risk_level': 'Low'
            }

            # Scan for suspicious patterns
            if self.deep_scan:
                file_info['suspicious_patterns'] = self.scan_file_patterns(file_path)

            # YARA scan
            if self.rules:
                matches = self.rules.match(str(file_path))
                if matches:
                    file_info['yara_matches'] = [
                        {'rule': match.rule, 'tags': match.tags}
                        for match in matches
                    ]

            # Determine risk level
            risk_score = 0
            if file_info['suspicious_patterns']:
                risk_score += len(file_info['suspicious_patterns'])
            if file_info['yara_matches']:
                risk_score += len(file_info['yara_matches']) * 2

            if risk_score > 5:
                file_info['risk_level'] = 'High'
            elif risk_score > 2:
                file_info['risk_level'] = 'Medium'

            self.results_queue.put(file_info)
            
            if file_info['risk_level'] != 'Low':
                self.logger.warning(
                    f"Suspicious file found: {file_path} "
                    f"(Risk Level: {file_info['risk_level']})"
                )

        except Exception as e:
            self.logger.error(f"Error scanning {file_path}: {str(e)}")

    def scan_directory(self):
        try:
            self.logger.info(f"Starting scan of {self.scan_path}")
            
            if self.scan_path.is_file():
                self.scan_file(self.scan_path)
            else:
                for root, _, files in os.walk(self.scan_path):
                    for file in files:
                        file_path = Path(root) / file
                        self.scan_file(file_path)
                        
        except Exception as e:
            self.logger.error(f"Error scanning directory: {str(e)}")

    def scan_processes(self):
        """Scan running processes for suspicious behavior"""
        try:
            for proc in psutil.process_iter(['pid', 'name', 'cmdline', 'connections']):
                try:
                    proc_info = proc.info
                    suspicious = False
                    reasons = []

                    # Check for suspicious network connections
                    if proc_info['connections']:
                        for conn in proc_info['connections']:
                            if conn.status == 'ESTABLISHED':
                                suspicious = True
                                reasons.append(f"Active connection to {conn.raddr}")

                    # Check for suspicious command line arguments
                    if proc_info['cmdline']:
                        cmdline = ' '.join(proc_info['cmdline'])
                        if any(pattern in cmdline.lower() for pattern in 
                              ['powershell', 'cmd.exe', 'nc', 'netcat']):
                            suspicious = True
                            reasons.append("Suspicious command line")

                    if suspicious:
                        self.results_queue.put({
                            'type': 'process',
                            'pid': proc_info['pid'],
                            'name': proc_info['name'],
                            'reasons': reasons,
                            'risk_level': 'Medium'
                        })

                except (psutil.NoSuchProcess, psutil.AccessDenied):
                    continue

        except Exception as e:
            self.logger.error(f"Error scanning processes: {str(e)}")

    def save_results(self, output_file):
        results = []
        while not self.results_queue.empty():
            results.append(self.results_queue.get())

        scan_results = {
            'timestamp': datetime.now().isoformat(),
            'scan_path': str(self.scan_path),
            'deep_scan': self.deep_scan,
            'total_files': len([r for r in results if 'path' in r]),
            'total_processes': len([r for r in results if 'pid' in r]),
            'findings': results
        }

        try:
            with open(output_file, 'w') as f:
                json.dump(scan_results, f, indent=4)
            self.logger.info(f"Results saved to {output_file}")
        except Exception as e:
            self.logger.error(f"Error saving results: {str(e)}")

    def print_summary(self):
        results = []
        while not self.results_queue.empty():
            results.append(self.results_queue.get())

        self.logger.info("\nScan Summary:")
        
        # File statistics
        files = [r for r in results if 'path' in r]
        self.logger.info(f"Total files scanned: {len(files)}")
        
        risk_levels = {'High': 0, 'Medium': 0, 'Low': 0}
        for result in files:
            risk_levels[result['risk_level']] += 1
        
        self.logger.info("\nRisk Level Distribution:")
        for level, count in risk_levels.items():
            self.logger.info(f"{level} Risk: {count} files")

        # Process statistics
        processes = [r for r in results if 'pid' in r]
        if processes:
            self.logger.info(f"\nSuspicious Processes Found: {len(processes)}")
            for proc in processes:
                self.logger.info(
                    f"PID {proc['pid']} ({proc['name']}): "
                    f"{', '.join(proc['reasons'])}"
                )

    def start_scan(self, output_file=None):
        self.scan_directory()
        if self.deep_scan:
            self.scan_processes()
        
        if output_file:
            self.save_results(output_file)
        
        self.print_summary()

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Trojan Detector")
    parser.add_argument("path", help="Directory or file path to scan")
    parser.add_argument("--rules", help="Path to YARA rules file")
    parser.add_argument("--deep", action="store_true",
                      help="Enable deep scan mode")
    parser.add_argument("--output", help="Output file for scan results")
    args = parser.parse_args()

    detector = TrojanDetector(args.path, args.rules, args.deep)
    detector.start_scan(args.output)
